

/* solitair.c */


/*------------------------------------------------------------------------
 * Tool for a complete brute force analysis of standard Solitaire
 *
 *   ooo         ...
 *   ooo         ...
 * ooooooo     .......
 * ooo.ooo ==> ...o...
 * ooooooo     .......
 *   ooo         ...
 *   ooo         ...
 *
 *------------------------------------------------------------------------
 * Solitaire Brute Force V1.0
 * Copyright (C) 2001 Holger Hasselbach <H.Hasselbach@galad.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *------------------------------------------------------------------------
 */


/*----------------------------------------------------------------------*/


/*
 * Thanks to a clear modular structure and a clean namespace even for
 * static filescope names, I can simply include all .c files together
 * (in arbitrary order) to one large project. It is of course not
 * recommended for larger projects, but IMO an excellent way for testing
 * and small closed projects like this.
 */


#include "mytypes.h"
#include "mymemory.c"
#include "mystring.c"

#include "wordcnt.c"
#include "field.c"
#include "hashtabs.c"
#include "hashblck.c"
#include "statgame.c"
#include "statstne.c"
#include "statsymm.c"
#include "stattree.c"
#include "stattabs.c"
#include "action.c"
#include "table.c"
#include "tabcell.c"

#include <stdlib.h>
#include <stdio.h>


/*----------------------------------------------------------------------*/


static tPSZ apz_Solitair_Tables[] =
  {
    "{Main results dec/hex}"
        "Rf({},{All-Tree},{Nrm-Tree})"
        "Rf({WinWays},Wax,Wnx)"
        "Rf({EndWays},++AWx,++NWx)"
        "Rf({Patterns},++APx,++NPx)",

    "{Patterns}"
        "Rf({Stones},Cy[{}],{Nrm},{All})"
        "Rs(Cy[SSNP],S+NP,S+AP)",

    "{Patterns with 0 moves}"
        "Rf({Stones},Cy[{}],{Nrm},{All},{EndWays},{EndWays})"
        "Rs(Cy[SSNM0],S+NM0,S+AM0,S+NW,S+AW)",

    "{Moves}"
        "Rf({Stones},Cy[{}],{Nrm},Cy[{}],{All})"
        "Rs(Cy[SSNS],S+NS,Cy[SSAS],S+AS)",

    "{Overview Nrm/All}"
        "Rf({Stones},{Patterns},{Pat0Moves},{Moves},{EndWays},{Patterns},{Pat0Moves},{Moves},{EndWays})"
        "Rs(S+NP,S+NM0,S+NS,S+NW,S+AP,S+AM0,S+AS,S+AW)",

    "{Patterns(Moves)}"
        "Rf({Moves},Cy[{}],{Nrm},Cy[{}],{All})"
        "Rm(Cy[+SNM],++NM,Cy[+SAM],++AM)",

    NULL
  };

/*---------------------------------------------*/

tSINT main (tVOID)
{
    tpsACTION psAction;
    tPSZ      pzTable = NULL;
    tpPSZ     ppzTab;
    tpFILE    pfOut;


    psAction = psActionCreate();


    vActionGameLoop(psAction,"  111  "
                             "  111  "
                             "1111111"
                             "1110111"
                             "1111111"
                             "  111  "
                             "  111  ");

    /*
    vActionGameLoop(psAction,"  010  "
                             "  010  "
                             "0000000"
                             "1100011"
                             "0000000"
                             "  010  "
                             "  010  ");
    */


    pfOut = fopen("results.htm","w");

    if (pfOut)
      {
        ppzTab = apz_Solitair_Tables;
        while (*ppzTab)
          {
            ppzActionTable(&pzTable,psAction,*ppzTab);
            fprintf(pfOut,"\n%s\n",pzTable);
            ppzTab++;
          }

        ppzActionSpecials(&pzTable,psAction);
        fprintf(pfOut,"<pre>\n%s</pre>\n",( pzTable ? pzTable : "" ));

        ppzStringClear(&pzTable);
        fclose(pfOut);
      }
    else
        printf("\n*** Error opening output file! ***\n\n");


    psActionDestroy(psAction);


    printf("\nAllc: %lu  Free: %lu\n\n", (unsigned long)cnMemoryAllocCount() , (unsigned long)cnMemoryFreeCount() );

    return (EXIT_SUCCESS);
}


/*----------------------------------------------------------------------*/
