

/* stattabs.c */


/*------------------------------------------------------------------------
 * Solitaire field statistics: Statistic table style defaults
 *------------------------------------------------------------------------
 * This version of stattabs is part of the Solitaire Brute Force tool.
 * Copyright (C) 2001 Holger Hasselbach <H.Hasselbach@galad.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *------------------------------------------------------------------------
 */


/*----------------------------------------------------------------------*/

#include "stattabs.h"
#include "mymemory.h"
#include "mystring.h"
#include "table.h"

/*----------------------------------------------------------------------*/


struct s_STATTABS
{
    tpsTABLE psTable;

    tCOUNT   cnStone;
    tCOUNT   cnSymm;
    tCOUNT   cnMove;

    tBOOL    bTitle;
    tBOOL    bHeader;
    tBOOL    bEnum;
    tBOOL    bEntry;

    tBOOL    bEntryAll;
    tBOOL    bEntryNrm;
    tBOOL    bEntryAdd;
    tBOOL    bEntrySum;
};

typedef struct s_STATTABS tsSTATTABS;

/*---------------------------------------------*/

static tVOID v_StatTabs_SetStyle (tpsSTATTABS psTabs);


/*----------------------------------------------------------------------*/


/*-----------------------------------------------
 * Local: Set table style
 *-----------------------------------------------
 */


struct s_STATTABSSTYLE
{
    tULONG ulColorBack;
    tULONG ulColorFont;
    tCHAR  cAlign;
    tBOOL  bBroad;
    tBOOL  bItalic;
};

typedef struct s_STATTABSSTYLE ts_STATTABSSTYLE;
typedef ts_STATTABSSTYLE       *tps_STATTABSSTYLE;

/*---------------------------------------------*/

static ts_STATTABSSTYLE as_StatTabs_Defaults[] =
  {
    { 0x00cccccc,0xff000000,'l',FALSE,FALSE },  /*  0 - Nothing    */
    { 0x00cccccc,0xff000000,'l',TRUE ,FALSE },  /*  1 - Title      */
    { 0x00cccccc,0xff000000,'c',TRUE ,FALSE },  /*  2 - Header     */
    { 0x00cccccc,0xff000000,'c',FALSE,FALSE },  /*  3 - Enum       */

    { 0x00cccccc,0xff000000,'r',FALSE,FALSE },  /*  4 - NonEnum    */
    { 0x00cccccc,0xff000000,'r',TRUE ,FALSE },  /*  5 - NonEnumSum */
    { 0x00cccccc,0xff000000,'r',FALSE,TRUE  },  /*  6 - NonAdd     */
    { 0x00cccccc,0xff000000,'r',TRUE ,TRUE  },  /*  7 - NonAddSum  */

    { 0x009999cc,0xff000000,'r',FALSE,FALSE },  /*  8 - AllEnum    */
    { 0x009999cc,0xff000000,'r',TRUE ,FALSE },  /*  9 - AllEnumSum */
    { 0x009999ff,0xff000000,'r',FALSE,FALSE },  /* 10 - AllAdd     */
    { 0x009999ff,0xff000000,'r',TRUE ,FALSE },  /* 11 - AllAddSum  */

    { 0x00cc9999,0xff000000,'r',FALSE,FALSE },  /* 12 - NrmEnum    */
    { 0x00cc9999,0xff000000,'r',TRUE ,FALSE },  /* 13 - NrmEnumSum */
    { 0x00ff9999,0xff000000,'r',FALSE,FALSE },  /* 14 - NrmAdd     */
    { 0x00ff9999,0xff000000,'r',TRUE ,FALSE }   /* 15 - NrmAddSum  */
  };

/*---------------------------------------------*/

static tVOID v_StatTabs_SetStyle (tpsSTATTABS psTabs)
{
    tps_STATTABSSTYLE psStyle;
    tCOUNT            cnStyle;

    cnStyle = 0;
    if (psTabs->bTitle)
        cnStyle = 1;
    else if (psTabs->bHeader)
        cnStyle = 2;
    else if (psTabs->bEnum)
        cnStyle = 3;
    else if (psTabs->bEntry)
      {
        cnStyle = 4;
        if (psTabs->bEntryAll)
            cnStyle = 8;
        else if (psTabs->bEntryNrm)
            cnStyle = 12;

        if (psTabs->bEntryAdd)
            cnStyle += 2;
        if (psTabs->bEntrySum)
            cnStyle += 1;
      }

    psStyle = &(as_StatTabs_Defaults[cnStyle]);

    vTableSetColor(psTabs->psTable,psStyle->ulColorBack,psStyle->ulColorFont);
    vTableSetAlign(psTabs->psTable,psStyle->cAlign);
    vTableSetStyle(psTabs->psTable,psStyle->bBroad,psStyle->bItalic);
}


/*----------------------------------------------------------------------*/


/*-----------------------------------------------
 * Create a new STATTABS
 *-----------------------------------------------
 */

tpsSTATTABS psStatTabsCreate (tVOID)
{
    tpsSTATTABS psTabs;

    psTabs = pvMemoryAllocObject(sSTATTABS);

    psTabs->psTable = psTableCreate(0,0);

    return (psTabs);
}


/*-----------------------------------------------
 * Destroy a STATTABS
 *-----------------------------------------------
 */

tpsSTATTABS psStatTabsDestroy (tpsSTATTABS psTabs)
{
    if (psTabs)
      {
        psTableDestroy(psTabs->psTable);

        pvMemoryFree(psTabs);
      }

    return (NULL);
}


/*-----------------------------------------------
 * Memory statistics
 *-----------------------------------------------
 */

tpsSTATTABS psStatTabsMemUsage (tpsSTATTABS psTabs, tpCOUNT pcnSizeof, tpCOUNT pcnBlocks)
{
    if (psTabs)
      {
        psTableMemUsage(psTabs->psTable,pcnSizeof,pcnBlocks);

        *pcnSizeof += cnMemorySizeofObject(sSTATTABS);
        *pcnBlocks += 1;
      }

    return (psTabs);
}


/*----------------------------------------------------------------------*/


/*-----------------------------------------------
 * Get and set parameter storage
 *-----------------------------------------------
 */

tVOID vStatTabsSetStone (tpsSTATTABS psTabs, tCOUNT cnStone)
{
    psTabs->cnStone = cnStone;
}

/*---------------------------------------------*/

tVOID vStatTabsSetSymm (tpsSTATTABS psTabs, tCOUNT cnSymm )
{
    psTabs->cnSymm = cnSymm;
}

/*---------------------------------------------*/

tVOID vStatTabsSetMove (tpsSTATTABS psTabs, tCOUNT cnMove )
{
    psTabs->cnMove = cnMove;
}

/*---------------------------------------------*/

tCOUNT cnStatTabsGetStone (tpsSTATTABS psTabs)
{
    return (psTabs->cnStone);
}

/*---------------------------------------------*/

tCOUNT cnStatTabsGetSymm (tpsSTATTABS psTabs)
{
    return (psTabs->cnSymm);
}

/*---------------------------------------------*/

tCOUNT cnStatTabsGetMove (tpsSTATTABS psTabs)
{
    return (psTabs->cnMove);
}


/*-----------------------------------------------
 * Initialize table setting
 *-----------------------------------------------
 */

tVOID vStatTabsSetStart (tpsSTATTABS psTabs)
{
    vTableSetPosition(psTabs->psTable,0,0,TRUE,FALSE);
}


/*-----------------------------------------------
 * Start next table row
 *-----------------------------------------------
 */

tVOID vStatTabsSetNextRow (tpsSTATTABS psTabs)
{
    vTableSetNextRow(psTabs->psTable,0);
}


/*-----------------------------------------------
 * Set style flags
 *-----------------------------------------------
 */


tVOID vStatTabsStyleNone (tpsSTATTABS psTabs)
{
    psTabs->bTitle  =
    psTabs->bHeader =
    psTabs->bEnum   =
    psTabs->bEntry  = FALSE;
}

/*---------------------------------------------*/

tVOID vStatTabsStyleTitle (tpsSTATTABS psTabs)
{
    vStatTabsStyleNone(psTabs);
    psTabs->bTitle = TRUE;
}

/*---------------------------------------------*/

tVOID vStatTabsStyleHeader (tpsSTATTABS psTabs)
{
    vStatTabsStyleNone(psTabs);
    psTabs->bHeader = TRUE;
}

/*---------------------------------------------*/

tVOID vStatTabsStyleEnum (tpsSTATTABS psTabs)
{
    vStatTabsStyleNone(psTabs);
    psTabs->bEnum = TRUE;
}

/*---------------------------------------------*/

tVOID vStatTabsStyleEntry (tpsSTATTABS psTabs)
{
    vStatTabsStyleNone(psTabs);
    psTabs->bEntry = TRUE;

    psTabs->bEntryAll =
    psTabs->bEntryNrm =
    psTabs->bEntryAdd =
    psTabs->bEntrySum = FALSE;
}

/*---------------------------------------------*/

tVOID vStatTabsStyleEntryAll (tpsSTATTABS psTabs)
{
    psTabs->bEntryAll = TRUE;
    psTabs->bEntryNrm = FALSE;
}

/*---------------------------------------------*/

tVOID vStatTabsStyleEntryNrm (tpsSTATTABS psTabs)
{
    psTabs->bEntryAll = FALSE;
    psTabs->bEntryNrm = TRUE;
}

/*---------------------------------------------*/

tVOID vStatTabsStyleEntryEnum (tpsSTATTABS psTabs)
{
    psTabs->bEntryAdd = FALSE;
}

/*---------------------------------------------*/

tVOID vStatTabsStyleEntryAdd (tpsSTATTABS psTabs)
{
    psTabs->bEntryAdd = TRUE;
}

/*---------------------------------------------*/

tVOID vStatTabsStyleEntrySum (tpsSTATTABS psTabs)
{
    psTabs->bEntrySum = TRUE;
}



/*-----------------------------------------------
 * Set next cell content
 *-----------------------------------------------
 */


tVOID vStatTabsSetCellText (tpsSTATTABS psTabs, tPSZc pzText)
{
    v_StatTabs_SetStyle(psTabs);
    vTableSetCellText(psTabs->psTable,pzText);
}

/*---------------------------------------------*/

tVOID vStatTabsSetCellWordCnt (tpsSTATTABS psTabs, tpsWORDCNT psCnt, tCOUNT cnIndex, tCHAR cFormat)
{
    tPSZ pzStrD = NULL,
         pzStrH = NULL;

    ppzWordCntToStr(&pzStrD,psCnt,cnIndex,10);
    ppzStringFormatNumber(&pzStrD,pzStrD,0,3,".","","");

    if (cFormat == 'x')
      {
        ppzWordCntToStr(&pzStrH,psCnt,cnIndex,16);
        ppzStringFormatNumber(&pzStrH,pzStrH,0,4,".","","");
        ppzStringConcatN(&pzStrD,3,pzStrD,"<br>",pzStrH);
        ppzStringClear(&pzStrH);
      }

    vStatTabsSetCellText(psTabs,pzStrD);
    ppzStringClear(&pzStrD);
}

/*---------------------------------------------*/

tVOID vStatTabsSetCellULong (tpsSTATTABS psTabs, tULONG ulVal)
{
    tPSZ pzStr = NULL;

    ppzStringULongToStr(&pzStr,ulVal,10);
    ppzStringFormatNumber(&pzStr,pzStr,0,3,".","","");
    vStatTabsSetCellText(psTabs,pzStr);
    ppzStringClear(&pzStr);
}


/*-----------------------------------------------
 * Get the table HTML string
 *-----------------------------------------------
 */

tpPSZ ppzStatTabsHtmlStr (tpPSZ ppzStr, tpsSTATTABS psTabs)
{
    vTableExtendRow(psTabs->psTable,0);

    return ( ppzTableHtmlStr(ppzStr,psTabs->psTable) );
}


/*----------------------------------------------------------------------*/


/*-----------------------------------------------
 * Search next frame start
 *-----------------------------------------------
 * NULL is not valid for pzStr.
 * Return is position with cBeg or a pointer to '\0'.
 * Thus not using the standard lib because of
 * NULL-return.
 *-----------------------------------------------
 */

tPSZc pzStatTabsStrPosNext (tPSZc pzStr, tCHAR cBeg)
{
    while ( *pzStr && (*pzStr != cBeg) )
        pzStr++;

    return (pzStr);
}


/*-----------------------------------------------
 * Search next position behind frame
 *-----------------------------------------------
 * NULL is not valid for pzStr.
 * The first char in pzStr is expected to be cBeg.
 * In this case, return is the position behind
 * cEnd. If cEnd is nonexistent, return is the
 * pointer to the string closing '\0'. The frame
 * is allowed to be nonexistent. When the first
 * char is not cBeg, return is the original pzStr.
 * cBeg=='\0' or cEnd=='\0' invokes nasal demons
 * and is checked out.
 *-----------------------------------------------
 */

tPSZc pzStatTabsStrPosBehind (tPSZc pzStr, tCHAR cBeg, tCHAR cEnd)
{
    if ( cBeg && cEnd && (*pzStr == cBeg) )
      {
        pzStr++;
        while ( *pzStr && (*pzStr != cEnd) )
            pzStr++;

        if (*pzStr)
            pzStr++;
      }

    return (pzStr);
}


/*-----------------------------------------------
 * Get next part before separator
 *-----------------------------------------------
 * NULL is not valid for pzStr.
 * Searches cSep. When found then the part from
 * start to cSep (exclusive) is stored in Part,
 * return is first position behind cSep. When not
 * found then the whole string is stored in Part,
 * return is pointer to '\0'. '\0' is allowed
 * for cSep, then return is pointer to '\0'.
 *-----------------------------------------------
 */

tPSZc pzStatTabsStrGetSep (tPSZc pzStr, tCHAR cSep, tpPSZ ppzPart)
{
    tPSZc pzBegin;

    pzBegin = pzStr;
    while ( *pzStr && (*pzStr != cSep) )
        pzStr++;

    ppzStringMiddle(ppzPart,pzBegin,0,pzStr - pzBegin);

    if (*pzStr)
        pzStr++;

    return (pzStr);
}


/*-----------------------------------------------
 * Get interior of frame
 *-----------------------------------------------
 * For pzStr, cBeg, cEnd and return, the same is
 * valid as in StrPosBehind.
 * pzInner is cleared if frame is nonexistent.
 *-----------------------------------------------
 */

tPSZc pzStatTabsStrGetInner (tPSZc pzStr, tCHAR cBeg, tCHAR cEnd, tpPSZ ppzInner)
{
    ppzStringClear(ppzInner);
    if ( cBeg && cEnd && (*pzStr == cBeg) )
        pzStr = pzStatTabsStrGetSep(pzStr + 1,cEnd,ppzInner);

    return (pzStr);
}


/*-----------------------------------------------
 * Get a number
 *-----------------------------------------------
 * NULL is not valid for pzStr.
 * Return is original pzStr with unchanged cnNumber
 * if there is no number. Else, return is the first
 * position behind the number. Overflow is not
 * detected.
 *-----------------------------------------------
 * strtoul not used for some unwanted properties.
 *-----------------------------------------------
 */

tPSZc pzStatTabsStrGetNumber (tPSZc pzStr, tpCOUNT pcnNumber)
{
    tPSZc  pzLoop;
    tCOUNT cnLoop;

    cnLoop = 0;
    pzLoop = pzStr;
    while ( (*pzLoop >= '0') && (*pzLoop <= '9') )
      {
        cnLoop = (cnLoop * 10) + (tCOUNT)(*pzLoop - '0');
        pzLoop++;
      }

    if (pzLoop != pzStr)
        *pcnNumber = cnLoop;

    return (pzLoop);
}


/*----------------------------------------------------------------------*/
